# samp for sampling
import pandas as pd
import math
import scipy.stats as stats
import itertools
import random


def to_xbar_freq(possible_outcome, repeat=1):
    xbar_all = []
    for aelem in itertools.product(possible_outcome, repeat=repeat):
        xbar_all.append(np.mean(aelem))

    xbar_df = pd.DataFrame({'xbar': xbar_all})
    xbar_freq = xbar_df.xbar.value_counts()
    xbar_freq = xbar_freq.to_frame().reset_index()
    xbar_freq = xbar_freq.sort_values(by="index")
    xbar_freq = xbar_freq.rename(columns={'index': 'x', 'xbar': 'freq'})
    tmp1 = xbar_freq.freq.sum()
    xbar_freq['prob'] = xbar_freq.freq / tmp1
    return xbar_freq


def sumprob(xfreq, bm=3, tp=4):
    tmpind1 = xfreq.x >= bm
    tmpind2 = xfreq.x <= tp
    return xfreq[tmpind1 & tmpind2].prob.sum()


def returnZ(x, mu, std):
    return (x - mu) / std


def check5(n, p):
    return n * p > 5 and n * (1 - p) > 5


def returnE(x1, x2):
    return x1 - x2


def returnStd(std1, std2, n1, n2):
    return math.sqrt((std1 ** 2) / n1 + (std2 ** 2) / n2)


def returnVar(std1, std2, n1, n2):
    return math.pow(returnStd(std1, std2, n1, n2), 2)


# Estimation


def con_level(x_bar, sigma, n, sig_level, toprint=True):
    a = sig_level
    con_coef = 1 - a
    z_value = stats.norm.ppf(1 - a / 2)
    sig_x_bar = sigma / math.sqrt(n)
    lcl = x_bar - z_value * sig_x_bar
    ucl = x_bar + z_value * sig_x_bar
    result = f"""{con_coef * 100:.1f}% Confidence Interval: [{lcl:.4f}, {ucl:.4f}]
Mean: {x_bar:.4f}
Sample Size: {n}
Z-Value: {z_value:.4f}
Sigma of X-Bar: {sig_x_bar:.4f}
    """
    if toprint:
        print(result)
    return {"lcl": lcl, "ucl": ucl, "x_bar": x_bar, "z_value": z_value, "sig_x_bar": sig_x_bar}


def bound(a, std, n):
    z_value = stats.norm.ppf(1 - a / 2)
    return z_value * std / math.sqrt(n)


def estimate_samp_size(bound, sig_level, sigma):
    a = sig_level
    z_value = stats.norm.ppf(1 - a / 2)
    n = math.pow(z_value * sigma / bound, 2)
    return n



import pandas as pd
import numpy as np
import math
import scipy.stats as stats


def p_value_method(x_mean, h0_mean, h0_std, samp_num, siglevel, option='left', precision=4):
    z_value = (x_mean - h0_mean) / (h0_std/(samp_num ** 0.5))
    opt = option.lower()[0]
    if opt == 'l':
        option = 'One-Tail Test (left tail)'
        p_value = stats.norm.cdf(z_value)
        zcv = stats.norm.ppf(1 - siglevel)
    elif opt == 'r':
        option = 'One-Tail Test (right tail)'
        p_value = stats.norm.sf(z_value)
        zcv = stats.norm.ppf(1 - siglevel)
    elif opt == 't':
        # two-tail test
        option = 'Two-Tail Test'
        p_value = (1 - stats.norm.cdf(z_value)) * 2
        zcv = stats.norm.ppf(1 - siglevel/2)

    if p_value >= 0 and p_value < 0.01:
        inter_p = 'Overwhelming Evidence'
    elif p_value >= 0.01 and p_value < 0.05:
        inter_p = 'Strong Evidence'
    elif p_value >= 0.05 and p_value < 0.1:
        inter_p = 'Weak Evidence'
    elif p_value >= .1:
        inter_p = 'No Evidence'

    result = f"""======= Analysis =======
Mean = {x_mean:.{precision}f}
Number of Observation = {samp_num:.{precision}f}
Hypothesized Mean (H0 Mean) = {h0_mean:.{precision}f}
Assumed Standard Devation = {h0_std:.{precision}f}
Significant Level (alpha) = {siglevel:.{precision}f}

Using {option}:
Difference = {x_mean - h0_mean}
z (Critical value) = {zcv:.{precision}f}
z (Observed value) = {z_value:.{precision}f}
p-value = {p_value:.{precision}f} ({inter_p})"""

    print(result)

    return zcv, p_value
